#include <cstdio>
#include <cstdlib>
#include <vector>
#include "points.h"
#include <GL/glut.h>
#define width 800
#define height 640
#define ESC 27 //десятичный код клавиши Esc
#define ENTER 13 //десятичный код клавиши Enter

using namespace std;

int mouse_x = 0, mouse_y = 0; //координаты точки при последнем нажатии левой кнопки мыши
bool enter_flag = false; //была ли нажата клавиша Enter
vector <Vertex> polygon, points; //здесь будут сохраняться координаты
Vertex *ptr; //вспомогательный указатель

void draw_dot () {
	if ( !points.empty() ) { //прорисовка вершин многоугольника
		glPointSize ( 5 ); //размер точки в пикселях (5х5 пикселей)
		glBegin ( GL_POINTS ); //тип примитива(точка)
		glColor3ub ( 0, 0, 0 ); //установка цвета рисования(RGB, черный)
		//прорисовываем точками все вершины
		for ( auto now : points )
			glVertex2i ( now.x, now.y );
		glEnd ();
		//была нажата клавиша Enter, построение многоугольника завершено
		if ( enter_flag ) {
			Vertex point; //точка, проверяемая на принадлежность многоугольнику
			glPointSize ( 9 );
			//проверяем квадрат 9x9, т.к. размер точки 9х9 пикселей
			//координаты точки содержатся в mouse_x, mouse_y
			for ( int i = mouse_x - 4; i <= mouse_x + 4; i++ ) {
				for ( int j = mouse_y - 4; j <= mouse_y + 4; j++ ) {
					point.x = i; point.y = j;
					if ( is_point_in_polygon ( polygon, point, width ) == in ||
							is_point_in_polygon( polygon, point, width ) == border ) {
						//если точка внутри или на границе, меняем цвет рисования на красный
						glColor3ub ( 255, 0, 0 );
						i = mouse_x + 5; //для выхода из внешнего цикла
						break;
					}
				}
			}
			glBegin ( GL_POINTS );
			//рисуем точку на месте последнего щелчка мыши
			glVertex2i ( mouse_x, mouse_y );
			glEnd ();
		}
	}
}

void draw_line () {
	if ( !polygon.empty() ) {
		glColor3ub ( 18, 31, 154 ); //синий
		glLineWidth ( 2 ); //толщина линии
		glBegin ( GL_LINES ); //тип примитива-линия
		//соединяем по две соседние вершины
		for ( unsigned i = 1; i < polygon.size(); i++ ) {
			glVertex2i ( polygon[ i - 1].x, polygon[i - 1].y );
			glVertex2i ( polygon[i].x, polygon[i].y );
		}
		if ( enter_flag ) { //построение завершено, соединяем первую и последнюю вершины
			glVertex2i ( polygon[0].x, polygon[0].y );
			glVertex2i ( polygon.back().x, polygon.back().y );
		}
		glEnd ();
	}
}

void reshape ( int w, int h ) {
	glViewport ( 0, 0, width, height ); //область вывода
	glLoadIdentity (); //замена текущей матрицы на единичную
	gluOrtho2D ( 0, width,  0, height ); //плоскость отсечения
	glMatrixMode ( GL_MODELVIEW ); //матрица преобразований
	glFlush();
}

//создание заголовка окна с координатами точки
void generate_window_title () {
	char *str;
	str = new char [50]; //выделение памяти под строку
	//формирование заголовка
	sprintf ( str, "x = %d, y = %d", mouse_x, mouse_y );
	glutSetWindowTitle ( str ); //установка заголовка
	delete [] str;
}

void display () {
	generate_window_title(); //обновление заголовка
	glClear (GL_COLOR_BUFFER_BIT); //очистка текущего буфера цвета
	draw_dot (); //прорисовка вершин и точки
	draw_line(); //прорисовка ребер
	glFlush(); //вывод на экран
}

void mouse ( int button, int state, int x, int y ) {
	if ( state == GLUT_DOWN ) //нажата кнопка
		switch ( button ) {
			//если левая кнопка
			case GLUT_LEFT_BUTTON:
				mouse_x = x; mouse_y = height - y; //запоминаем новые координаты точки
				if ( !enter_flag ) { //добавляем новые координаты(если не был нажат Enter)
					ptr = new Vertex ( mouse_x, mouse_y );
					polygon.push_back(*ptr);
					points.push_back(*ptr);
					delete ptr;
				}
				break;
			//если правая кнопка
			case GLUT_RIGHT_BUTTON:
				//сбрасываем координаты
				mouse_x = 0; mouse_y = 0;
				//освобождаем память
				polygon.clear();
				points.clear();
				//сбрасываем флаг Enter
				enter_flag = false;
				break;
		}
	glutPostRedisplay (); //обратный вызов функции перерисовки окна
}

void keyboard ( unsigned char key, int x, int y ) {
	switch ( key ) {
		case ESC: //если был нажат esc, выходим из приложения
			exit (ESC);
			break;
		case ENTER: //если enter
			enter_flag = true; //устанавливаем флаг
			glutPostRedisplay (); //вызываем функцию перерисовки для отображения последнего ребра
			break;
	}
}

int main ( int argc, char** argv ) {
	glutInit( &argc, argv ); //инициализация OpenGL Utility Toolkit
	glutInitDisplayMode( GLUT_SINGLE | GLUT_RGB ); //режим окна(одиночный буфер и RGB-палитра)
	glutInitWindowSize ( width, height ); //установка размеров окна(ширина и высота в пикселях)
	glutInitWindowPosition ( 200, 30 );//положение окна относительно левого верхнего угла экрана
	glutCreateWindow ( "" ); //создание окна
	glutReshapeFunc ( reshape );  //перерисовка окна при измении его размеров или положения
	glutDisplayFunc ( display );//инициализация функции, отвечающей за рисование в окне
	glutMouseFunc ( mouse ); //обработка мыши
	glutKeyboardFunc ( keyboard ); //обработка клавиатуры
	glClearColor ( 1.0, 1.0, 1.0, 1.0 ); //цвет фона(RGBA, белый)
	glutMainLoop (); //вход в главный цикл
	return 0;
}
