#include "points.h"
#include <vector>
using namespace std;


//[ {ab} x {ac} ]
double cross_prod ( Vertex a, Vertex b, Vertex c ) {
	double x1 = b.x - a.x, y1 = b.y - a.y,
		x2 = c.x - a.x, y2 = c.y - a.y;
	return ( x1 * y2 - x2 * y1 );
}

//( {ab}, {ac} )
double scalar_prod ( Vertex a, Vertex b, Vertex c ) {
	double x1 = b.x - a.x, y1 = b.y - a.y,
		x2 = c.x - a.x, y2 = c.y - a.y;
	return ( x1 * x2 + y1 * y2 );
}

//точка c лежит на ребре ab, если она лежит на одной прямой с ab(векторное произведение = 0),
//и c лежит между a и b(скалярное произведение <= 0; = 0, когда c совпадает с a или c b)
bool is_point_on_edge ( Vertex a, Vertex b, Vertex c ) {
	double prod1 = cross_prod ( a, b, c ),
		prod2 = scalar_prod ( c, a, b );
	return ( prod1 == 0 && prod2 <= 0 ) ? true : false;
}

//пересекаются ли отрезки ab и cd
bool is_edge_cross ( Vertex a, Vertex b, Vertex c, Vertex d ) {
	double prod1, prod2, prod3, prod4;
	prod1 = cross_prod ( a, b, c );
	prod2 = cross_prod ( a, b, d );
	prod3 = cross_prod ( d, c, a );
	prod4 = cross_prod ( d, c, b );
	//отрезки ab и cd пересекаются, если c и d лежат по разные стороны от ab,
	//a и b лежат по разные стороны от cd
	bool is_cross = (prod1 * prod2 < 0 && prod3 * prod4 < 0);
	//надо проверить, не принадлежат ли концы одного отрезка другому
	//в этом случае одно(или несколько) векторных произведений обратятся в ноль
	bool cond1, cond2, cond3, cond4, cond1_1, cond2_1, cond3_1, cond4_1, is_one_inside_other;
	cond1 = is_point_on_edge ( a, b, c );
	cond2 = is_point_on_edge ( a, b, d );
	cond3 = is_point_on_edge ( c, d, a );
	cond4 = is_point_on_edge ( c, d, b );
	//проверка, что только один конец одного из отрезков принадлежит другому
	//таким образом, из рассмотрения исключаюся ребра, лежащие на луче
	cond1_1 = ( cond1 && ! ( cond2 || cond3 || cond4 ) );
	cond2_1 = ( cond2 && ! ( cond1 || cond3 || cond4 ) );
	cond3_1 = ( cond3 && ! ( cond1 || cond2 || cond4 ) );
	cond4_1 = ( cond4 && ! ( cond1 || cond2 || cond3 ) );
	is_one_inside_other = ( cond1_1 || cond2_1 || cond3_1 || cond4_1 );

	return ( is_cross || is_one_inside_other) ? true : false;
}

//выбор точки на луче, проходящем через проверяемую точку point; в качестве такой точки
//выбрана крайняя точка окна, лежащая на одной горизонтальной прямой с point
Vertex point_on_ray ( Vertex point, int scr_width ) {
	Vertex ray ( scr_width, point.y );
	return ray;
}

//положение точки относительно многоугольника
point_position is_point_in_polygon ( vector <Vertex> polygon, Vertex begin, int scr_width ) {
	Vertex first, second; //вершины многоугольника
	//получение второй точки, лежащей на луче
	Vertex end = point_on_ray ( begin, scr_width );
	unsigned count = 0; //кол-во пересечений с лучом
	if ( polygon.empty() )
		return out; //не было передано ни одной вершины
	if ( polygon.size() == 1 ) //была передана одна вершина
		return ( ( polygon[0].x == begin.x && polygon[0].y == begin.y ) ? border : out );
	if ( polygon.size() == 2 ) { //был передан отрезок
		second.x = polygon[1].x; second.y = polygon[1].y;
		first.x = polygon[0].x; first.y = polygon[0].y;
		return ( is_point_on_edge ( first, second, begin ) ? border : out );
	}
	//был передан по крайней мере треугольник
	for ( unsigned i = 1; i <= polygon.size(); i++ ) {
		if ( i < polygon.size() ) {
			second.x = polygon[i].x; second.y = polygon[i].y;
			first.x = polygon[i - 1].x; first.y = polygon[i - 1].y;
		}
		else { //обрабатываем последнее ребро(последняя и первая вершины)
			second.x = polygon[0].x; second.y = polygon[0].y;
			first.x = polygon.back().x; first.y = polygon.back().y;
		}
		//принадлежит ли точка ребру
		if ( is_point_on_edge ( first, second, begin ) )
			return border; //точка на границе
		//пересекается ли ребро с лучом(при этом исключаются ребра, лежащие на луче)
		if ( is_edge_cross ( first, second, begin, end ) )
			//определяем нижнюю вершину по y
			//ребро не лежит на одной прямой с горизонтальным лучом (из предыдущего условия)
			//если выбрать не горизонтальный луч, надо дополнительно рассмотреть горизонтальные ребра
			if ( ( first.y > second.y && second.y < begin.y ) ||
					( first.y < second.y && first.y < begin.y ) )
					count++; //нижний конец ребра ниже луча
	}
	if ( count % 2 )
		return in; //нечетное кол-во пересечений с горизонтальным лучом
	return out;
}



